@@ -2,18 +2,41 @@ import asyncio
 from fbrp import life_cycle
 from fbrp.process import ProcDef
 import a0
-import argparse
 import asyncio
 import contextlib
+import pathlib
 import psutil
 import json
 
 
+# psutil as_dict() does not produce a json-serializable dict.
+# https://github.com/giampaolo/psutil/issues/967
+#
+# When converted to a dict, fields like memory_info generate
+#     [11984896, 31031296, ...]
+# instead of
+#     {"rss": 11984896, "vms": 31031296, ...}
+# Losing field names.
+#
+# We cannot use a custom JSONEncoder, since psutil objects, like memory_info,
+# inherit from tuples.
+#
+# This is a workaround.
+def _walk_asdict(obj):
+    if hasattr(obj, "_asdict"):
+        return _walk_asdict(obj._asdict())
+    if isinstance(obj, dict):
+        return {k: _walk_asdict(v) for k, v in obj.items()}
+    if isinstance(obj, list):
+        return [_walk_asdict(v) for v in obj]
+    return obj
+
+
 class BaseLauncher:
     def __init__(self):
         pass
 
-    async def run(self, name: str, proc_def: ProcDef, args: argparse.Namespace):
+    async def run(self, name: str, proc_def: ProcDef):
         raise NotImplementedError("Launcher hasn't implemented run!")
 
     def get_pid(self):
@@ -47,7 +70,7 @@ class BaseLauncher:
 
             try:
                 proc = psutil.Process(pid)
-                out.pub(json.dumps(proc.as_dict()))
+                out.pub(json.dumps(_walk_asdict(proc.as_dict())))
             except psutil.NoSuchProcess:
                 pass
 
@@ -56,10 +79,11 @@ class BaseRuntime:
     def __init__(self):
         pass
 
-    def _build(self, name: str, proc_def: ProcDef, args: argparse.Namespace):
+    def asdict(self, root: pathlib.Path):
+        raise NotImplementedError("Runtime hasn't implemented asdict!")
+
+    def _build(self, name: str, proc_def: ProcDef):
         raise NotImplementedError("Runtime hasn't implemented build!")
 
-    def _launcher(
-        self, name: str, proc_def: ProcDef, args: argparse.Namespace
-    ) -> BaseLauncher:
+    def _launcher(self, name: str, proc_def: ProcDef) -> BaseLauncher:
         raise NotImplementedError("Runtime hasn't implemented a launcher!")
